use Test::More tests => 1;

BEGIN {
    use_ok( 'PDL::Graphics::OpenGL::Perl::OpenGL' );
}

diag( "Testing PDL::Graphics::OpenGL::Perl::OpenGL $PDL::Graphics::OpenGL::Perl::OpenGL::VERSION, Perl $], $^X" );
